//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: DebugCommand.cpp,v 1.64 2008/01/16 18:31:14 gmf Exp $
//

#include "DebugCommand.hpp"

#include <xqilla/context/DynamicContext.hpp>
#include <xqilla/simple-api/XQilla.hpp>
#include <xqilla/simple-api/XQQuery.hpp>

#include <db_cxx.h>
#include "ScopedDbt.hpp"
#include "NameID.hpp"

#include "UTF8.hpp"
#include "QueryContext.hpp"
#include "PrintDataItemTree.hpp"
#include "OperationContext.hpp"
#include "Document.hpp"
#include "Container.hpp"
#include "Key.hpp"
#include "DbWrapper.hpp"
#include "query/QueryExecutionContext.hpp"
#include "Globals.hpp"
#include "optimizer/ASTReplaceOptimizer.hpp"
#include "optimizer/QueryPlanOptimizer.hpp"
#include "optimizer/ImpliedSchemaGenerator.hpp"
#include "optimizer/QueryPlanGenerator.hpp"
#include "dataItem/DbXmlFactoryImpl.hpp"
#include "dataItem/DbXmlConfiguration.hpp"

#include <xqilla/framework/XPath2MemoryManagerImpl.hpp>
#include <xqilla/context/DynamicContext.hpp>
#include <xqilla/exceptions/XQException.hpp>
#include <xqilla/exceptions/QueryInterruptedException.hpp>

using namespace DbXml;
using namespace std;

string DebugCommand::getCommandName() const
{
	return "debug";
}

string DebugCommand::getBriefHelp() const
{
	return "Debug command -- internal use only";
}

string DebugCommand::getMoreHelp() const
{
	return "";
}

void DebugCommand::execute(Args &args, Environment &env)
{
	if(args.size() != 2) {
		throw CommandException("Wrong number of arguments");
	}

	try {
		ReferenceMinder minder;
		CacheDatabaseMinder dbMinder(new CacheDatabaseMinderImpl(env.db()));
		Transaction *txn = 0;
		if (env.txn())
			txn = *env.txn();
		CompileInfo ci;
		ProjectionInfo pi;
		DbXmlConfiguration conf(env.context(), txn, &ci);
		conf.setMinder(&minder);
		QueryExecutionContext qec((QueryContext&)env.context(), false);
		conf.setQueryExecutionContext(&qec);
		conf.setFlags(DBXML_LAZY_DOCS);
		conf.setDbMinder(dbMinder);
		conf.setProjectionInfo(&pi);

		DynamicContext *context = XQilla::createContext(XQilla::XQUERY_UPDATE, &conf, Globals::defaultMemoryManager);

		XQilla xqilla;
		AutoDelete<XQQuery> query(xqilla.parse(UTF8ToXMLCh(args[1]).str(), context, 0,
						  XQilla::NO_STATIC_RESOLUTION | XQilla::NO_ADOPT_CONTEXT));

		OutputTree *before = new OutputTree("Initial tree", context);
		StaticResolver *sr = new StaticResolver(context, before);
		OutputTree *afterStatic = new OutputTree("After Static Resolution", context, sr);
		ASTReplaceOptimizer *rep = new ASTReplaceOptimizer(minder, context, afterStatic);
		OutputTree *afterRep = new OutputTree("After AST Replace", context, rep);
		StaticTyper *st = new StaticTyper(context, afterRep);
		OutputTree *afterST = new OutputTree("After Static Typing", context, st);
		ImpliedSchemaGenerator *isg = new ImpliedSchemaGenerator(context, afterST);
		OutputTree *afterIsg = new OutputTree("After ImpliedSchemaGenerator", context, isg);
		QueryPlanGenerator *qpGen = new QueryPlanGenerator(context, afterIsg);
		OutputTree *afterQP = new OutputTree("After Query Plan Generation", context, qpGen);
		QueryPlanOptimizer *qpo = new QueryPlanOptimizer(OptimizationContext::RESOLVE_INDEXES, context, afterQP);
		OutputTree *afterQPO = new OutputTree("After QueryPlan Optimisation Phase 1 (RESOLVE_INDEXES)", context, qpo);
		StaticTyper *st2 = new StaticTyper(context, afterQPO);
		OutputTree *afterST2 = new OutputTree("After Static Typing (2)", context, st2);
		QueryPlanOptimizer *qpo2 = new QueryPlanOptimizer(OptimizationContext::REARRANGE, context, afterST2);
		OutputTree *afterQPO2 = new OutputTree("After QueryPlan Optimisation Phase 2 (REARRANGE)", context, qpo2);
		QueryPlanOptimizer *ao = new QueryPlanOptimizer(OptimizationContext::ALTERNATIVES, context, afterQPO2);
		OutputTree *afterAO = new OutputTree("After QueryPlan Optimisation Phase 3 (ALTERNATIVES)", context, ao);
		QueryPlanOptimizer *qpo3 = new QueryPlanOptimizer(OptimizationContext::ADD_STEPS, context, afterAO);
		OutputTree *afterQPO3 = new OutputTree("After QueryPlan Optimisation Phase 4 (ADD_STEPS)", context, qpo3);
		QueryPlanOptimizer *qpo4 = new QueryPlanOptimizer(OptimizationContext::MAKE_PREDICATES, context, afterQPO3);
		OutputTree *afterQPO4 = new OutputTree("After QueryPlan Optimisation Phase 5 (MAKE_PREDICATES)", context, qpo4);
		QueryPlanOptimizer *qpo5 = new QueryPlanOptimizer(OptimizationContext::REMOVE_REDUNDENTS, context, afterQPO4);
		OutputTree *afterQPO5 = new OutputTree("After QueryPlan Optimisation Phase 6 (REMOVE_REDUNDENTS)", context, qpo5);
		StaticTyper *st3 = new StaticTyper(context, afterQPO5);
		OutputTree *afterST3 = new OutputTree("After Static Typing (3)", context, st3);
		ProjectionSchemaAdjuster *psa = new ProjectionSchemaAdjuster(context, afterST3);

		AutoDelete<Optimizer> optimizer(psa);

		optimizer->startOptimize(query);

		cout << "Implied schemas: " << endl;
		ImpliedSchemaNode::Vector::const_iterator i;
		for(i = isg->getRoots().begin();
		    i != isg->getRoots().end(); ++i) {
			cout << (*i)->toString();
		}
		for(i = psa->getRoots().begin();
		    i != psa->getRoots().end(); ++i) {
			cout << (*i)->toString();
		}

		if(!env.documentProjection())
			conf.setProjectionInfo(0);

		AutoDelete<DynamicContext> context2(context->createDynamicContext(Globals::defaultMemoryManager));
		Result result(query->execute(context2));

		cout << "Results:\n";
		int count = 0;
		Item::Ptr item;
		while((item = result->next(context2)) != NULLRCP) {
			if(env.sigBlock().isInterrupted()) {
				env.sigBlock().reset();
				throw CommandException("Debug interrupted");
			}

			++count;
// 			cout << XMLChToUTF8(item->asString(context2)).str() << endl;
		}

		cout << count << " results returned\n";

	}
	catch(const QueryInterruptedException &e) {
		env.sigBlock().reset();
		throw CommandException("Debug interrupted");
	}
	catch(const XQException &e) {
		throw CommandException(XMLChToUTF8(e.getError()).str());
	}
	catch(XERCES_CPP_NAMESPACE_QUALIFIER XMLException &e) {
		throw CommandException(XMLChToUTF8(e.getMessage()).str());
	}
	catch(DbException &e) {
		throw XmlException(e);
	}
	catch(XmlException &e) {
		if(e.getExceptionCode() == XmlException::OPERATION_INTERRUPTED)
			env.sigBlock().reset();
		throw CommandException(e.what());
	}
}

