/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2007,2008 Oracle.  All rights reserved.
 *
 * $Id: ModifyTest.java,v 1.9 2008/03/26 15:17:49 lauren Exp $
 */

package dbxmltest;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlManager;
import com.sleepycat.dbxml.XmlModify;
import com.sleepycat.dbxml.XmlQueryContext;
import com.sleepycat.dbxml.XmlQueryExpression;
import com.sleepycat.dbxml.XmlResults;
import com.sleepycat.dbxml.XmlTransaction;
import com.sleepycat.dbxml.XmlUpdateContext;
import com.sleepycat.dbxml.XmlValue;

public class ModifyTest {
  private static final String cname = "testData.dbxml";

  private TestConfig hp = null;
  private XmlManager mgr = null;
  private XmlContainer cont = null;
  private XmlTransaction txn = null;

  private static String docName = "testDoc.xml";


  @BeforeClass
  public static void setupClass() {
    System.out.println("Begin test XmlModifyTest");
  }

  @AfterClass
  public static void tearDownClass() {
    System.out.println("Finished test XmlModifyTest");
  }

  @Before
  public void setUp() throws Throwable {
    TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
    hp = new TestConfig(XmlTestRunner.getEnvironmentType(),
        XmlTestRunner.isNodeContainer(), 
        XmlTestRunner.getEnvironmentPath());
    mgr = hp.createManager();
    cont = hp.createContainer(cname, mgr);
  }

  @After
  public void tearDown() throws Throwable {
    if (txn != null)
      txn.commit();
    hp.closeContainer(cont);
    hp.closeManager(mgr);
    hp.closeEnvironment();
    TestConfig.fileRemove(null);
  }

  /*
   * Test addAppendStep(XmlQueryExpression selectionExpr, int type, String
   * name, String content)
   * 
   */
  @Test
  public void testAddAppendStep() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlModify mo = mgr.createModify();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    // Element
    try {
      mo.addAppendStep(qe, XmlModify.Element, "node3", "I am node3");
    } catch (XmlException e) {
      throw e;
    }

    // Attribute
    try {
      mo.addAppendStep(qe, XmlModify.Attribute, "atr3", "testatr");
    } catch (XmlException e) {
      throw e;
    }

    // Comment
    try {
      mo.addAppendStep(qe, XmlModify.Comment, "", "this is comment");
    } catch (XmlException e) {
      throw e;
    }

    // ProcessingInstruction
    try {
      mo.addAppendStep(qe, XmlModify.ProcessingInstruction, "do", "read");
    } catch (XmlException e) {
      throw e;
    }

    // Text
    try {
      mo.addAppendStep(qe, XmlModify.Text, "", "test text");
    } catch (XmlException e) {
      throw e;
    }

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument><node1"
        + " attr1=\"an attribute node\"/><node2 atr3=\"test"
        + "atr\"><node3>I am node3</node3><!--this is"
        + " comment--><?do read?>test text</node2></sampleDocument>");
    qe.delete();
    mo.delete();
  }

  /*
   * addAppendStep(XmlQueryExpression selectionExpr, int type, String name,
   * String content, int location)
   * 
   */
  @Test
  public void testAppendStep_location() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }


    XmlModify mo = mgr.createModify();

    // Attribute
    try {
      mo.addAppendStep(qe, XmlModify.Attribute, "atr3", "testatr", 1);
    } catch (XmlException e) {
      throw e;
    }

    // Comment
    try {
      mo.addAppendStep(qe, XmlModify.Comment, "", "this is comment", 1);
    } catch (XmlException e) {
      throw e;
    }

    // ProcessingInstruction
    try {
      mo.addAppendStep(qe, XmlModify.ProcessingInstruction, "do", "read",
          1);
    } catch (XmlException e) {
      throw e;
    }

    // Text
    try {
      mo.addAppendStep(qe, XmlModify.Text, "", "test text", 1);
    } catch (XmlException e) {
      throw e;
    }

    // Element
    try {
      mo.addAppendStep(qe, XmlModify.Element, "node3", "I am node3", 2);
    } catch (XmlException e) {
      throw e;
    }

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument>"
        + "<node1 attr1=\"an attribute node\""
        + "/><node2 atr3=\"testatr\"><!--this is "
        + "comment-->test text<node3>I am node3</node3><?do"
        + " read?></node2></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test addAppendStep(XmlQueryExpression selectionExpr, int type, String
   * name, XmlResults resultContent, int location)
   * 
   */
  @Test
  public void testAppendStep_Result() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    // Attribute
    XmlResults result = mgr.createResults();
    XmlValue vl = new XmlValue("testatr");
    result.add(vl);
    try {
      mo.addAppendStep(qe, XmlModify.Attribute, "atr3", result, 1);
    } catch (XmlException e) {
      throw e;
    }
    result.delete();

    // Comment
    result = mgr.createResults();
    vl = new XmlValue("this is comment");
    result.add(vl);
    try {
      mo.addAppendStep(qe, XmlModify.Comment, "", result, 1);
    } catch (XmlException e) {
      throw e;
    }
    result.delete();

    // ProcessingInstruction
    result = mgr.createResults();
    vl = new XmlValue("read");
    result.add(vl);
    try {
      mo.addAppendStep(qe, XmlModify.ProcessingInstruction, "do", result,
          1);
    } catch (XmlException e) {
      throw e;
    }
    result.delete();

    // Text
    result = mgr.createResults();
    vl = new XmlValue("test text");
    result.add(vl);
    try {
      mo.addAppendStep(qe, XmlModify.Text, "", result, 1);
    } catch (XmlException e) {
      throw e;
    }
    result.delete();

    // Element
    result = mgr.createResults();
    vl = new XmlValue("I am node3");
    result.add(vl);
    try {
      mo.addAppendStep(qe, XmlModify.Element, "node3", result, 2);
    } catch (XmlException e) {
      throw e;
    }
    result.delete();

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument>"
        + "<node1 attr1=\"an attribute node\""
        + "/><node2 atr3=\"testatr\"><!--this is "
        + "comment-->test text<node3>I am node3</node3><?do"
        + " read?></node2></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for addInsertAfterStep(XmlQueryExpression selectionExpr, int type,
   * String name, String content)
   * 
   */
  @Test
  public void testAddInsertAfterStep() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    // Element
    try {
      mo.addInsertAfterStep(qe, XmlModify.Element, "node3", "I am node3");
    } catch (XmlException e) {
      throw e;
    }

    // Attribute
    try {
      mo.addInsertAfterStep(qe, XmlModify.Attribute, "atr3", "testatr");
    } catch (XmlException e) {
      throw e;
    }

    // Comment
    try {
      mo.addInsertAfterStep(qe, XmlModify.Comment, "", "this is comment");
    } catch (XmlException e) {
      throw e;
    }

    // ProcessingInstruction
    try {
      mo.addInsertAfterStep(qe, XmlModify.ProcessingInstruction, "do",
      "read");
    } catch (XmlException e) {
      throw e;
    }

    // Text
    try {
      mo.addInsertAfterStep(qe, XmlModify.Text, "", "test text");
    } catch (XmlException e) {
      throw e;
    }

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }

    assertEquals(retDoc2.getContentAsString(), "<sampleDocument "
        + "atr3=\"testatr\"><node1 attr1=\"an attribute "
        + "node\"/><node2/>test text<?do read?><!--this "
        + "is comment--><node3>I am node3</node3></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for addInsertAfterStep(XmlQueryExpression selectionExpr, int type,
   * String name, XmlResults resultContent)
   * 
   */
  @Test
  public void testAddInsertAfterStep_Result() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    // Attribute
    XmlResults result = mgr.createResults();
    XmlValue vl = new XmlValue("testatr");
    result.add(vl);
    try {
      mo.addInsertAfterStep(qe, XmlModify.Attribute, "atr3", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // Comment
    result = mgr.createResults();
    vl = new XmlValue("this is comment");
    result.add(vl);
    try {
      mo.addInsertAfterStep(qe, XmlModify.Comment, "", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // ProcessingInstruction
    result = mgr.createResults();
    vl = new XmlValue("read");
    result.add(vl);
    try {
      mo.addInsertAfterStep(qe, XmlModify.ProcessingInstruction, "do",
          result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // Text
    result = mgr.createResults();
    vl = new XmlValue("test text");
    result.add(vl);
    try {
      mo.addInsertAfterStep(qe, XmlModify.Text, "", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // Element
    result = mgr.createResults();
    vl = new XmlValue("I am node3");
    result.add(vl);
    try {
      mo.addInsertAfterStep(qe, XmlModify.Element, "node3", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument "
        + "atr3=\"testatr\"><node1 attr1=\"an attribute "
        + "node\"/><node2/><node3>I am node3</node3>test "
        + "text<?do read?><!--this is comment--></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for addInsertBeforeStep(XmlQueryExpression selectionExpr, int type,
   * String name, String content)
   * 
   */
  @Test
  public void testAddInsertBeforeStep() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    // Element
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Element, "node3",
      "I am node3");
    } catch (XmlException e) {
      throw e;
    }

    // Attribute
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Attribute, "atr3", "testatr");
    } catch (XmlException e) {
      throw e;
    }

    // Comment
    try {
      mo
      .addInsertBeforeStep(qe, XmlModify.Comment, "",
      "this is comment");
    } catch (XmlException e) {
      throw e;
    }

    // ProcessingInstruction
    try {
      mo.addInsertBeforeStep(qe, XmlModify.ProcessingInstruction, "do",
      "read");
    } catch (XmlException e) {
      throw e;
    }

    // Text
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Text, "", "test text");
    } catch (XmlException e) {
      throw e;
    }

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }

    assertEquals(retDoc2.getContentAsString(), "<sampleDocument "
        + "atr3=\"testatr\"><node1 attr1=\"an attribute "
        + "node\"/><node3>I am node3</node3><!--this is "
        + "comment--><?do read?>test text<node2/></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for addInsertBeforeStep(XmlQueryExpression selectionExpr, int type,
   * String name, XmlResults resultContent)
   * 
   */
  @Test
  public void testAddInsertBeforeStep_Result() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    // Attribute
    XmlResults result = mgr.createResults();
    XmlValue vl = new XmlValue("testatr");
    result.add(vl);
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Attribute, "atr3", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // Comment
    result = mgr.createResults();
    vl = new XmlValue("this is comment");
    result.add(vl);
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Comment, "", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // ProcessingInstruction
    result = mgr.createResults();
    vl = new XmlValue("read");
    result.add(vl);
    try {
      mo.addInsertBeforeStep(qe, XmlModify.ProcessingInstruction, "do",
          result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // Text
    result = mgr.createResults();
    vl = new XmlValue("test text");
    result.add(vl);
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Text, "", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    // Element
    result = mgr.createResults();
    vl = new XmlValue("I am node3");
    result.add(vl);
    try {
      mo.addInsertBeforeStep(qe, XmlModify.Element, "node3", result);
    } catch (XmlException e) {
      throw e;
    }finally{
	result.delete();
    }

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument "
        + "atr3=\"testatr\"><node1 attr1=\"an attribute "
        + "node\"/><!--this is comment--><?do read?>test "
        + "text<node3>I am node3</node3><node2/></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * addRemoveStep(XmlQueryExpression selectionExpr)
   * 
   */
  @Test
  public void testAddRemoveStep() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node1";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    try {
      mo.addRemoveStep(qe);
    } catch (XmlException e) {
      throw e;
    }
    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(),
    "<sampleDocument><node2/></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for addRenameStep(XmlQueryExpression selectionExpr,String newName)
   * 
   */
  @Test
  public void testRenameStep() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2 /></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node1";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    try {
      mo.addRenameStep(qe, "newNode");
    } catch (XmlException e) {
      throw e;
    }
    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument><"
        + "newNode attr1=\"an attribute node\"/>"
        + "<node2/></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for addUpdateStep(XmlQueryExpression selectionExpr,String newName)
   * 
   */
  @Test
  public void testAddUpdateStep() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2>old text</node2></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();

    try {
      mo.addUpdateStep(qe, "replacement text");
    } catch (XmlException e) {
      throw e;
    }
    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      mo.execute(txn, docValue, qc, uc);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      mo.execute(docValue, qc, uc);
      retDoc2 = cont.getDocument(docName);
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument><node1"
        + " attr1=\"an attribute node\"/><node2>replacement"
        + " text</node2></sampleDocument>");

    qe.delete();
    mo.delete();
  }

  /*
   * Test for delete()
   * 
   */
  @Test
  public void testDelete() throws Throwable {

  }

  /*
   * Test for execute(XmlResults toModify, XmlQueryContext context, XmlUpdateContext uc) 
   * 
   */
  @Test
  public void testExecute() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 attr1=\"an attribute"
      + " node\" /><node2>old text</node2></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();
    mo.addUpdateStep(qe, "replacement text");

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    XmlResults result = mgr.createResults();
    try{
	if (hp.isTransactional()) {
	    retDoc = cont.getDocument(txn, docName);
	    docValue = new XmlValue(retDoc);
	    result.add(docValue);
	    mo.execute(txn, result, qc, uc);
	    retDoc2 = cont.getDocument(txn, docName);
	} else {
	    retDoc = cont.getDocument(docName);
	    docValue = new XmlValue(retDoc);
	    result.add(docValue);
	    mo.execute(result, qc, uc);
	    retDoc2 = cont.getDocument(docName);
	}
    }catch(XmlException e){
	throw e;
    }finally{
	result.delete();
	mo.delete();
	qe.delete();
    }
    assertEquals(retDoc2.getContentAsString(), "<sampleDocument><node1"
        + " attr1=\"an attribute node\"/><node2>replacement"
        + " text</node2></sampleDocument>");

    //test QUERY_EVALUATION_ERROR exception 	
    XmlModify mo2 = mgr.createModify();
    XmlResults result2 = mgr.createResults();
    XmlQueryExpression qe2 = null;
    XmlDocument delDoc;
    XmlValue docValue2;
    try {
      if (hp.isTransactional()) {
        qe2 = mgr.prepare(txn, "/", qc);
        mo2.addRemoveStep(qe2);
        delDoc = cont.getDocument(txn, docName);
        docValue2 = new XmlValue(delDoc);
        result2.add(docValue2);
        mo2.execute(txn, docValue2, qc, uc);
      } else {
        qe2 = mgr.prepare("/", qc);
        mo2.addRemoveStep(qe2);
        delDoc = cont.getDocument(docName);
        docValue2 = new XmlValue(delDoc);
        result2.add(docValue2);
        mo2.execute(docValue2, qc, uc);
      }
      fail("execute succeeded where it should have failed.");
    } catch (XmlException e) {
      assertNotNull(e.getMessage());
      assertEquals(e.getErrorCode(), XmlException.QUERY_EVALUATION_ERROR);
    }finally{
	qe2.delete();
	result2.delete();
	mo2.delete();
    }
  }

  /*
   * Test for execute(XmlValue toModify, XmlQueryContext context, XmlUpdateContext uc) 
   * 
   */
  @Test
  public void testExecute_Val() throws Throwable {
    XmlDocument doc = mgr.createDocument();
    XmlUpdateContext uc = mgr.createUpdateContext();
    doc.setName(docName);
    String docString = "<sampleDocument><node1 "
      + "attr1=\"an attribute node\" /><node2>old text</node2></sampleDocument>";

    doc.setContent(docString);
    cont.putDocument(doc, uc);

    String query = "/sampleDocument/node2";

    XmlQueryContext qc = mgr.createQueryContext();
    XmlQueryExpression qe;
    if (hp.isTransactional()) {
      txn = mgr.createTransaction();
      qe = mgr.prepare(txn, query, qc);
    } else {
      qe = mgr.prepare(query, qc);
    }

    XmlModify mo = mgr.createModify();
    mo.addUpdateStep(qe, "replacement text");

    XmlDocument retDoc;
    XmlDocument retDoc2;
    XmlValue docValue;
    try{
    if (hp.isTransactional()) {
      retDoc = cont.getDocument(txn, docName);
      docValue = new XmlValue(retDoc);
      assertEquals(mo.execute(txn, docValue, qc, uc), 1);
      retDoc2 = cont.getDocument(txn, docName);
    } else {
      retDoc = cont.getDocument(docName);
      docValue = new XmlValue(retDoc);
      assertEquals(mo.execute(docValue, qc, uc), 1);
      retDoc2 = cont.getDocument(docName);
    }
    }catch(XmlException e){
	throw e;
    }finally{
	mo.delete();
	qe.delete();
    }

    assertEquals(retDoc2.getContentAsString(), "<sampleDocument><node1"
        + " attr1=\"an attribute node\"/><node2>replacement"
        + " text</node2></sampleDocument>");

    //test QUERY_EVALUATION_ERROR exception 

    XmlModify mo2 = mgr.createModify();
    XmlQueryExpression qe2 = null;
    try {
      if (hp.isTransactional()) {
        qe2 = mgr.prepare(txn, "/", qc);
        mo2.addRemoveStep(qe2);
        mo2.execute(txn, docValue, qc, uc);
      } else {
        qe2 = mgr.prepare("/", qc);
        mo2.addRemoveStep(qe2);
        mo2.execute(docValue, qc, uc);
      }
      fail("execute succeeded where it should have failed.");
    } catch (XmlException e) {
      assertNotNull(e.getMessage());
      assertEquals(e.getErrorCode(), XmlException.QUERY_EVALUATION_ERROR);
    }finally{
	qe2.delete();
	mo2.delete();
    }
  }
}
